#include "Engine.h"
#include "SDL_KeyMap.h"
float maxAnisotropy=0;
bool gAnistropy = false;
bool gVsync = true;
bool gAntiAliasing = false;
int gAASamples = 2;

Engine* gpEng = NULL;

float gOScroller = 640;
float gScroller = 640;
char scroller[] = { "PRESS PAUSE TO CHANGE MOUSE SENSITIVITY... CONTROLS ARE WASD OR ARROWPAD TO MOVE AROUND SPACE OR LCTRL TO JUMP...  MOUSE L ACTIVATES BLUE PORTAL MOUSE R ACTIVATES YELLOW PORTAL...  ITS BEEN FUN.. WELL DONE EVERYONE THAT ENTERED LD14... PEACE X-OUT"}  ;

bool Engine::InitializeExtensions()
{
	gLog.OutPut("\n[Initializing Extensions]\n");
	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		gLog.OutPut("Failed To Init GLEW!\n");
		return false;
	}
	gLog.OutPut("GLEW Ready!\n");


	//vertex programs
	gLog.OutPut("Testing GL_ARB_vertex_program support.\n");
	if (!glewIsSupported("GL_ARB_vertex_program"))
	{
		gLog.OutPut("GL_ARB_vertex_program not supported!\n");
		return false;
	}
	gLog.OutPut("GL_ARB_vertex_program supported!\n");

	//fragment programs
	gLog.OutPut("Testing GL_ARB_fragment_program support.\n");
	if (!glewIsSupported("GL_ARB_fragment_program"))
	{
		gLog.OutPut("GL_ARB_fragment_program not supported!\n");
		return false;
	}
	gLog.OutPut("GL_ARB_fragment_program supported!\n");


	

	gLog.OutPut("Complete...\n\n");
	return true;
}
bool Engine::LoadShader(char* name, char* szVp, char* szFp, int& outShdr)
{
	gLog.OutPut("Compilling shader: ");
	gLog.OutPut(name);
	gLog.OutPut(".\n");
	unsigned int vp,fp;
	if(!(fp = mShdrMgr.CreateShader(GL_FRAGMENT_SHADER_ARB,szFp)))
		return false;
	if(!(vp = mShdrMgr.CreateShader(GL_VERTEX_SHADER_ARB,szVp)))
		return false;
	outShdr = mShdrMgr.CreateProgram();
	mShdrMgr.AttachShader(vp);
	mShdrMgr.AttachShader(fp);
	mShdrMgr.LinkProgram();
	mShdrMgr.BindProgram(0);
	return true;
}
Engine::Engine(int w, int h, bool fscreen)
{

	gpEng = this;
	gLog.SetLogState(1);

	if(!UTIL_SDL::InitSDL("LD14",w,h,32,fscreen,gAntiAliasing?gAASamples:0))
		return;
	bHiSpecs = InitializeExtensions();
	if(!bHiSpecs)
		gLog.OutPut("Using low-sys-specs render path!\n");

	glClear(GL_COLOR_BUFFER_BIT);
	SDL_GL_SwapBuffers();
	winWidth = w;
	winHeight = h;
	bTextInput = false;
	bCfgJoystick = false;
	mbPaused = false;
	mbQuit = false;
	mfMouseSensi = 0.5f;
	LoadInput();

	SDL_WM_GrabInput(SDL_GRAB_ON);

	InitDevil();

	portal1ID = 0;
	portal2ID = 1;
	glGenTextures(1,&portal1ID);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, portal1ID);
	UTIL_GL::TextureFilter(GL_LINEAR, GL_LINEAR);
	UTIL_GL::TextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,256,256,0,GL_RGB,GL_UNSIGNED_BYTE,NULL);
	glDisable(GL_TEXTURE_2D);
	glGenTextures(1,&portal2ID);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, portal2ID);
	UTIL_GL::TextureFilter(GL_LINEAR, GL_LINEAR);
	UTIL_GL::TextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,256,256,0,GL_RGB,GL_UNSIGNED_BYTE,NULL);
	glDisable(GL_TEXTURE_2D);

	glGenTextures(1,&iSceneID);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, iSceneID);
	UTIL_GL::TextureFilter(GL_LINEAR, GL_LINEAR);
	UTIL_GL::TextureWrap(GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,1024,512,0,GL_RGB,GL_UNSIGNED_BYTE,NULL);
	glDisable(GL_TEXTURE_2D);

	if(!mFont1.LoadFont("Fonts.psd","FontShadow.psd"))
		return;
	if(!(xhairID = UTIL_GL::Image::LoadImage("Data/xhair.png",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return;
	if(bHiSpecs)
	{
		if(!(portalID = UTIL_GL::Image::LoadImage("Data/portal.png",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return;
	}
	else
	{
		if(!(portalID = UTIL_GL::Image::LoadImage("Data/Ring_low.tga",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
			return;
	}
	if(!(ringID = UTIL_GL::Image::LoadImage("Data/ring.png",GL_NEAREST, GL_NEAREST, GL_REPEAT, GL_REPEAT)))
		return;


	if(bHiSpecs)
	{
		if(!LoadShader("pass through", vp_pass, fp_pass, mShdrPass))
			return;
		if(!LoadShader("pass through2", vp_pass, fp_pass2, mShdrPass2))
			return;
		if(!LoadShader("fadegrey", vp_pass, fp_fadegrey, mShdrGrey))
			return;
		mShdrMgr.BindProgram(0);
	}

	iStage = 0;
	if(!LoadStage())
		return;

	mTimer.Reset();

	Pump();

}

void Engine::InitDevil()
{
	gLog.OutPut("\n[Initializing Devil]\n");
	ilInit();
	iluInit();
	ilutInit();
	gLog.OutPut("Complete...\n\n");
}
Engine::~Engine()
{
//	SaveInput();
//  Opcode::CloseOpcode();
}

void Engine::Pump()
{
	mMouseMotion = Vec2(0,0);
	mTimer.Reset();
	char FPS[64];
	accumulator=0;
	SDL_Event event;
	while(!mbQuit)
	{
		mTimer.Update();
		accumulator += mTimer.GetDelta();
		while(accumulator>=TIME_STEP)
		{
			while(SDL_PollEvent(&event)>0)
			{

				switch(event.type)
				{
					case SDL_KEYDOWN:
						if(bTextInput)
							TextInput(event.key.keysym);
						else
						KeyStateAlltered(event.key.keysym.sym,true);
					break;
					case SDL_KEYUP:
					KeyStateAlltered(event.key.keysym.sym,false);
					break;
					//joystick buttons
					case SDL_JOYBUTTONDOWN:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, true);				
						}
						break;
					case SDL_JOYBUTTONUP:
						{
						//	if(!bJoystickSupported)
						//		break;
							int b = event.jbutton.button;
							if(b<20)
								KeyStateAlltered(JOYSTICK_BUTTON+b, false);				
						}
						break;//*/
					//joystick motion
					case SDL_JOYAXISMOTION:
					//	if(!bJoystickSupported)
					//		break;
						CheckJoystickMotion(event);
						break;//*/

					case SDL_QUIT:
						mbQuit = true;
					break;

					case SDL_MOUSEBUTTONDOWN:
					KeyStateAlltered(event.button.button,true);
					break;
					case SDL_MOUSEBUTTONUP:
					KeyStateAlltered(event.button.button,false);
					break;

					case SDL_MOUSEMOTION:
						mMouseMotion.x = (float)(event.motion.xrel);
						mMouseMotion.y = (float)(event.motion.yrel);
						break;
				}
			}
			SDL_GetMouseState(&MouseX,&MouseY);
			Update();
			miInternalClock++;

			KeysPostUpdate();
			accumulator-=TIME_STEP;
		}

		const float interp = mInterp = accumulator/TIME_STEP;
		
		mbPaused?
			Render(1):
			Render(interp);
		//UTIL_GL::GetGLError();

			UTIL_GL::GL2D::SetOrtho(640,480);
			glDisable(GL_DEPTH_TEST);
			mFont1.DisableShadow();
			mFont1.SetColor(1,1,1,1);
			mFont1.SetScale(1,1);
			mFont1.SetBlendMode(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
			sprintf(FPS,"%i",mTimer.GetFPS());
			mFont1.SetAlignment(ALIGN_LEFT);
			mFont1.Print(FPS,15,15);
			glDisable(GL_BLEND);

		SDL_GL_SwapBuffers();
		SDL_Delay(1);
	}
}

void Engine::Update()
{
	if(KEY_QUIT.state == JUST_PRESSED)
	{
		mbQuit =true;
	}
	if(KEY_PAUSE.state == JUST_PRESSED)
		mbPaused = !mbPaused;
	if(mbPaused)
	{
		if(KEY_MOUSE1.state == JUST_PRESSED || KEY_MOUSE3.state == JUST_PRESSED)
			mbPaused = false;
		if(KEY_LEFT.state == PRESSED)
			mfMouseSensi -= 0.01f;
		if(KEY_RIGHT.state == PRESSED)
			mfMouseSensi += 0.01f;

		if(mfMouseSensi< 0.1)
			mfMouseSensi = 0.1;
		if(mfMouseSensi> 0.99)
			mfMouseSensi = 0.99;
		return;
	}

	gOScroller = gScroller;
	gScroller -= 3.5f;
	if(gScroller<-5000)
	{
		gScroller = gOScroller = 640;
	}


	if(bStartGame)
	{
		iStartTime+=5;
		if(iStartTime>250)
			bStartGame = false;
		mMouseMotion = Vec2(0,0);
		return;
	}
	if(bDead)
	{
		iDeathTime+=2;
		if(iDeathTime>250)
		{
			bDead = false;
			bStartGame = true;
			iStartTime = 0;
			if(!LoadStage())
				mbQuit = true;
		}
		mMouseMotion = Vec2(0,0);
		return;
	}
	if(bWonStage)
	{
		iWonTime++;
		if(iWonTime>250)
		{
			iStage++;
			bStartGame = true;
			iStartTime = 0;
			if(!LoadStage())
				mbQuit = true;
		}
		mMouseMotion = Vec2(0,0);
		return;
	}


	mPlrVel.y += mfPlrGravity;

	if(!bPortalFriction)
		mPlrVel *= 0.915f;
	else
	{
		mPlrVel.y *= 0.945f;
	}


	mOPlrAngles = mPlrAngles;
	mPlrAngles.x -= mMouseMotion.x * mfMouseSensi;
	mPlrAngles.y += mMouseMotion.y * mfMouseSensi;
	if(mPlrAngles.x < 0)
	{
		mPlrAngles.x += 360;
		mOPlrAngles.x += 360;
	}
	if(mPlrAngles.x>=360)
	{
		mPlrAngles.x -= 360;
		mOPlrAngles.x -= 360;
	}
	if(mPlrAngles.y > 89)
	{
		mPlrAngles.y = 89;
		mOPlrAngles.y = 89;
	}
	if(mPlrAngles.y < -89)
	{
		mPlrAngles.y = -89;
		mOPlrAngles.y = -89;
	}
	mOPlrFwd = mPlrFwd;
	mOPlrRight = mPlrRight;
	mOPlrUp = mPlrUp;
	Vec3 tmpAngle = Vec3(mPlrAngles.x, 0,0);
	tmpAngle.MakeVectors(mPlrFwd, mPlrRight, mPlrUp);
	//mMouseMotion = Vec2(0,0);
	mMouseMotion *= 0.475f;

	if(KEY_FORWARD.state == PRESSED)
	{
		mPlrVel += Vec3(mPlrFwd.x, 0, mPlrFwd.z) * 0.085f;
	}
	if(KEY_BACK.state == PRESSED)
	{
		mPlrVel -= Vec3(mPlrFwd.x, 0, mPlrFwd.z) * 0.075f;
	}
	if(KEY_LEFT.state == PRESSED)
	{
		mPlrVel += Vec3(mPlrRight.x, 0, mPlrRight.z) * 0.054f;
	}
	if(KEY_RIGHT.state == PRESSED)
	{
		mPlrVel -= Vec3(mPlrRight.x, 0, mPlrRight.z) * 0.054f;
	}

	if(bOnGround)
	{
		if(KEY_JUMP.state == JUST_PRESSED)
		{
			mPlrVel.y = -1.9f;
		}
	}
	mPlrAngles.MakeVectors(mPlrFwd, mPlrRight, mPlrUp);

	mOPlrPos = mPlrPos;
	mPlrPos += mPlrVel;
	if(bOnPlatform)
		if(pRidingPlatform)
			if(pRidingPlatform->bMoving)
				mPlrPos += pRidingPlatform->mNormal * PLATFORM_SPEED;

	mfPlrGravity += 0.0056f;

	//falling collision.. break down into a number of steps
	bOnGround = false;
	bOnPlatform = false;
	pRidingPlatform = NULL;
	if(mPlrVel.y>=0)
	{
		int length = (mPlrVel.y * MAP_SCALE)+1;
		for(int fall = 0; fall<length; fall++)
		{
			Vec3 p = mOPlrPos + ((mPlrPos-mOPlrPos) * ((float)fall/(float)length));
			//check we are landing on a platform
			bool onPlat = false;
			std::list<Platform>::iterator plat;
			for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
			{
				if((*plat).CheckPointInPlat(p+Vec3(0,1,0) * PLAYER_HEIGHT))
				{
					//gLog.OutPut("landed on plat!\n");
					if(mPlrVel.y>DEATH_SPEED)
					{
						bDead = true;
						iDeathTime = 0;
						mOPlrPos = mPlrPos;
						mOPlrAngles = mPlrAngles;
					}
					onPlat = true;
					mfPlrGravity = 0;
					mPlrVel.y = 0;
					mPlrPos.y = (int)(p.y/MAP_SCALE)+2;
					mPlrPos.y *= MAP_SCALE;
					mPlrPos.y -= PLAYER_HEIGHT;
					mOPlrPos.y = mPlrPos.y;
					bOnGround = true;
					bPortalFriction = false;
					bOnPlatform = true;
					pRidingPlatform = &(*plat);
					break;
				}
			}
			if(onPlat)
				break;



			int tile=0;
			if((tile=GetTileState(p+Vec3(0,1,0) * PLAYER_HEIGHT)))
			{
				if(mPlrVel.y>DEATH_SPEED)
				{
					bDead = true;
					iDeathTime = 0;
					mOPlrPos = mPlrPos;
					mOPlrAngles = mPlrAngles;
				}
				mfPlrGravity = 0;
				mPlrVel.y = 0;
				mPlrPos.y = (int)(p.y/MAP_SCALE)+2;
				mPlrPos.y *= MAP_SCALE;
				mPlrPos.y -= PLAYER_HEIGHT;
				mOPlrPos.y = mPlrPos.y;
				bOnGround = true;
				bPortalFriction = false;
				if(tile == EXIT_TILE)
				{
					bWonStage = true;
					iWonTime = 0;
					mOPlrAngles = mPlrAngles;
					mOPlrPos = mPlrPos;
					mMouseMotion = Vec2(0,0);
				}
				break;
			}
		}
	}

	if(mPlrVel.y<0)
	{
		int length = (-mPlrVel.y * MAP_SCALE)+1;
		for(int fall = 0; fall<length; fall++)
		{
			Vec3 p = mOPlrPos + ((mPlrPos-mOPlrPos) * ((float)fall/(float)length));

			bool bumpedHeadonPlat = false;
			std::list<Platform>::iterator plat;
			for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
			{
				if((*plat).CheckPointInPlat(p))
				{
					bumpedHeadonPlat = true;
					break;
				}
			}

			if(GetTileState(p) || bumpedHeadonPlat)
			{
				mfPlrGravity = 0.05f;
				mPlrVel.y = -mPlrVel.y;
				break;
			}
		}
	}
	//movement collision, broken into steps
	{
//		int length = (-mPlrVel.y * MAP_SCALE)+1;
//		for(int fall = 0; fall<length; fall++)
//		{
//			Vec3 p = mOPlrPos + ((mPlrPos-mOPlrPos) * ((float)fall/(float)length));
			//check see if we try to walk into a platform
//			bool collided = false;
			for(int n=0; n<3; n++)
			{
				if(n==1)
					continue;

				Vec3 v = Normalize(mPlrVel*MAP_SCALE);
				Vec3 dir(0,0,0);
				dir[n] = v[n]>0?ceil(v[n]):floor(v[n]);
				if(GetTileState(mPlrPos + dir * 5) || GetTileState(mPlrPos + dir * 5 + Vec3(0,PLAYER_HEIGHT-1,0)) )
				{
					mPlrVel[n] = 0;
					mPlrPos[n] = mOPlrPos[n];
//					collided = true;
				}
				//check see if we try to walk into a platform
				std::list<Platform>::iterator plat;
				for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
				{
					if((*plat).CheckPointInPlat(mPlrPos + dir * 5) || (*plat).CheckPointInPlat(mPlrPos + dir * 5 + Vec3(0,PLAYER_HEIGHT-1,0)))
					{
						mPlrVel[n] = 0;
						mPlrPos[n] = mOPlrPos[n];
						break;
					}
				}

			}
//			if(collided)
//				break;
//		}
	}

	//if near the red portal, warp to the blue
	if(iPortalDelayTime>0)
		iPortalDelayTime--;
	if(iPortalDelayTime==0)
	{
		if(bRenderPortal1 && bRenderPortal2 && mPlrVel.Length()>0 && Dot(Normalize(mPlrVel),mPortal2Normal)<0.5)
		if(Distance(mPlrPos,mPortal2Pos)<10 || Distance(mPlrPos+Vec3(0,1,0)*PLAYER_HEIGHT,mPortal2Pos)<10 || Distance(mPlrPos+Vec3(0,1,0)*(PLAYER_HEIGHT*0.5),mPortal2Pos)<10)
		{
			if(mPortal1Normal.y>0)
				mPlrPos = mPortal1Pos + mPortal1Normal * 5 - Vec3(0,1,0) * ((PLAYER_HEIGHT * 0.5)-1);
			else
				mPlrPos = mPortal1Pos + mPortal1Normal * 5 - Vec3(0,1,0) * ((PLAYER_HEIGHT *0.5)+1);
			mOPlrPos = mPlrPos;
			Vec3 v = mPortal1Normal * (mPlrVel.Length());
			if(mPortal1Normal.y<0 || mPortal1Normal.y>0)
			{
				v += Vec3(mPlrVel.x, 0, mPlrVel.z);
				v += mPortal2Normal;
			}
			bPortalFriction = true;
			if(mPortal1Normal.y<0)
				mfPlrGravity = -mfPlrGravity * 0.5f;
			else if(mPortal1Normal.y == 0)
			{
				if(mPlrVel.y < 0)
				{
					v.y += mPlrVel.y;
				}
				//rotate the player to face
				float angle=0;
				if(mPortal1Normal.x == 1)
					angle = 0;
				else if(mPortal1Normal.z == 1)
					angle = 90;
				else if(mPortal1Normal.x == -1)
					angle = 180;
				else if(mPortal1Normal.z == -1)
					angle = 270;
				float angle2=0;
				if(mPortal2Normal.x == 1)
					angle2 = 0;
				else if(mPortal2Normal.z == 1)
					angle2 = 90;
				else if(mPortal2Normal.x == -1)
					angle2 = 180;
				else if(mPortal2Normal.z == -1)
					angle2 = 270;

				float adif = mPlrAngles.x - angle2;
				mPlrAngles.x = angle+adif+180;
				mOPlrAngles.x = angle+adif+180;
			}
				mfPlrGravity = 0;
			mPlrVel = v;
			iPortalDelayTime = 25;
		}
	}
	if(iPortalDelayTime==0)
	{
		//if near blue, warp to red
		if(bRenderPortal1 && bRenderPortal2 && mPlrVel.Length()>0 && Dot(Normalize(mPlrVel),mPortal1Normal)<0.5)
		if(Distance(mPlrPos,mPortal1Pos)<10 || Distance(mPlrPos+Vec3(0,1,0)*PLAYER_HEIGHT,mPortal1Pos)<10 || Distance(mPlrPos+Vec3(0,1,0)*(PLAYER_HEIGHT*0.5),mPortal1Pos)<10)
		{
			if(mPortal2Normal.y>0)
				mPlrPos = mPortal2Pos + mPortal2Normal * 5 - Vec3(0,1,0) * ((PLAYER_HEIGHT * 0.5)-1);
			else
				mPlrPos = mPortal2Pos + mPortal2Normal * 5 - Vec3(0,1,0) * ((PLAYER_HEIGHT *0.5)+1);
			mOPlrPos = mPlrPos;
			Vec3 v = mPortal2Normal * (mPlrVel.Length());
			if(mPortal2Normal.y<0 || mPortal2Normal.y>0)
			{
				v += Vec3(mPlrVel.x, 0, mPlrVel.z);
				v += mPortal2Normal;
			}
			bPortalFriction = true;
			if(mPortal2Normal.y<0)
				mfPlrGravity = -mfPlrGravity * 0.5f;
			else if(mPortal2Normal.y == 0)
			{
				if(mPlrVel.y < 0)
				{
					v.y += mPlrVel.y;
				}
				//rotate the player to face
				float angle=0;
				if(mPortal2Normal.x == 1)
					angle = 0;
				else if(mPortal2Normal.z == 1)
					angle = 90;
				else if(mPortal2Normal.x == -1)
					angle = 180;
				else if(mPortal2Normal.z == -1)
					angle = 270;
				float angle2=0;
				if(mPortal1Normal.x == 1)
					angle2 = 0;
				else if(mPortal1Normal.z == 1)
					angle2 = 90;
				else if(mPortal1Normal.x == -1)
					angle2 = 180;
				else if(mPortal1Normal.z == -1)
					angle2 = 270;

				float adif = mPlrAngles.x - angle2;
				mPlrAngles.x = angle+adif+180;
				mOPlrAngles.x = angle+adif+180;
			}
				mfPlrGravity = 0;
			mPlrVel = v;
			iPortalDelayTime = 25;
		}
	}

	if(KEY_MOUSE1.state == JUST_PRESSED)
	{
		Vec3 hit, n;
		int tile;
		if((tile=ShootRay(mPlrPos,mPlrFwd,hit,n)))
		{
			if(tile<EXIT_TILE)
			if(Distance(hit,mPortal2Pos)>15 || !bRenderPortal2)
			if(n.y <0 || n.y>0 || !GetTileState(hit+Vec3(0,1,0)*(PLAYER_HEIGHT*0.5)) &&
				!GetTileState(hit-Vec3(0,1,0)*(PLAYER_HEIGHT*0.5)) )
			{
				mPortal1Pos = hit;
				mPortal1Normal = n;
				bRenderPortal1 = true;
			}

			if(tile>=EXIT_TILE)
			{
/*				char sz[64];
				sprintf(sz,"shot a non portal tile %i!\n", tile);
				gLog.OutPut(sz);*/
			}
		}
	}
	if(KEY_MOUSE3.state == JUST_PRESSED)
	{
		Vec3 hit, n;
		int tile;
		if((tile=ShootRay(mPlrPos,mPlrFwd,hit,n)))
		{
			if(tile<EXIT_TILE)
			if(Distance(hit,mPortal1Pos)>15 || !bRenderPortal1)
			if(n.y <0 || n.y>0 || !GetTileState(hit+Vec3(0,1,0)*(PLAYER_HEIGHT*0.5)) &&
				!GetTileState(hit-Vec3(0,1,0)*(PLAYER_HEIGHT*0.5)) )
			{
				mPortal2Pos = hit;
				mPortal2Normal = n;
				bRenderPortal2 = true;
			}
			if(tile>=EXIT_TILE)
			{

			}
		}
	}

	if(!mPlatforms.empty())
	{
		std::list<Platform>::iterator plat;
		for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
		{
			(*plat).Update();
		}
	}
}

void Engine::Render(const float interp)
{
	Vec3 p;
	Vec3 f;
	Vec3 a = UTIL_Misc::Interpolate(mPlrAngles, mOPlrAngles, interp);

	if(bRenderPortal1 && bHiSpecs)
	{
		f = mPortal2Normal;
		p = mPortal2Pos;
		glViewport(0,0,256,256);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		UTIL_GL::GL3D::SetPerspective(65,1.3,1,4000);
		if(mPortal2Normal.y == 0)
			gluLookAt(p.x,p.y,p.z, p.x+f.x,p.y+f.y,p.z+f.z, 0,-1,0);
		else 
			gluLookAt(p.x,p.y,p.z, p.x+f.x,p.y+f.y,p.z+f.z, -1,0,0);
		RenderMap(interp);
		UTIL_GL::BindTexture(GL_TEXTURE_2D, portal1ID);
		glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,256,256);
		glDisable(GL_TEXTURE_2D);
	}
	if(bRenderPortal2 && bHiSpecs)
	{
		f = mPortal1Normal;
		p = mPortal1Pos;
		glViewport(0,0,256,256);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		UTIL_GL::GL3D::SetPerspective(65,1.3,1,4000);
		if(mPortal1Normal.y == 0)
			gluLookAt(p.x,p.y,p.z, p.x+f.x,p.y+f.y,p.z+f.z, 0,-1,0);
		else 
			gluLookAt(p.x,p.y,p.z, p.x+f.x,p.y+f.y,p.z+f.z, -1,0,0);
		RenderMap(interp);
		UTIL_GL::BindTexture(GL_TEXTURE_2D, portal2ID);
		glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,256,256);
		glDisable(GL_TEXTURE_2D);
	}

	glViewport(0,0,640,480);
	UTIL_GL::GL3D::SetPerspective(65,1.3,1,4000);
	glClear(GL_DEPTH_BUFFER_BIT);
	p = UTIL_Misc::Interpolate(mPlrPos, mOPlrPos, interp) - Vec3(0,0.5f,0);
	a.MakeVectors(f, Vec3(0,0,0), Vec3(0,0,0));

	float u = 0;
	if(bDead)
	{
		float time = iDeathTime;
		if(time>100)
			time == 100;
		float fall = UTIL_Misc::Interpolate(0,PLAYER_HEIGHT-4,(float)time/100);
		u = UTIL_Misc::Interpolate(0,1,(float)time/100);
		p.y -= fall;
	}
	gluLookAt(p.x,p.y,p.z, p.x+f.x,p.y+f.y,p.z+f.z, u,-1,u);
	RenderMap(interp);

	if(bRenderPortal1)
	{
		Vec3 r, u;
		if(mPortal1Normal.y <0)
		{
			r = Vec3(0,0,1);
			u = Vec3(1,0,0);
		}
		else if(mPortal1Normal.y >0)
		{
			r = Vec3(0,0,-1);
			u = Vec3(1,0,0);
		}
		else if(mPortal1Normal.x <0)
		{
			r = Vec3(0,0,-1);
			u = Vec3(0,1,0);
		}
		else if(mPortal1Normal.x >0)
		{
			r = Vec3(0,0,1);
			u = Vec3(0,1,0);
		}
		else if(mPortal1Normal.z <0)
		{
			r = Vec3(1,0,0);
			u = Vec3(0,1,0);
		}
		else if(mPortal1Normal.z >0)
		{
			r = Vec3(-1,0,0);
			u = Vec3(0,1,0);
		}
		if(bHiSpecs)
		{
			Vec2 Scale(0.1+sin(mTimer.GetTime()*3) * 0.1, 0.1+cos(mTimer.GetTime()*3) * 0.1);
			if(bRenderPortal2)
			{

				mShdrMgr.BindProgram(mShdrPass);
				mShdrMgr.PassUniform2fv("scale",1,Scale.v);
				UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				mShdrMgr.PassSampler2D("tex",0,portal1ID);
				mShdrMgr.PassSampler2D("tex2",1,portalID);
				glBegin(GL_QUADS);
				glMultiTexCoord2f(GL_TEXTURE0,1,1); 
				glMultiTexCoord2f(GL_TEXTURE1,1,1); glVertex3fv((mPortal1Pos - r * 8 - u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,0,1); 
				glMultiTexCoord2f(GL_TEXTURE1,0,1); glVertex3fv((mPortal1Pos + r * 8 - u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,0,0); 
				glMultiTexCoord2f(GL_TEXTURE1,0,0); glVertex3fv((mPortal1Pos + r * 8 + u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,1,0); 
				glMultiTexCoord2f(GL_TEXTURE1,1,0); glVertex3fv((mPortal1Pos - r * 8 + u * 8).v);
				glEnd();
				glDisable(GL_TEXTURE_2D);
				mShdrMgr.BindProgram(0);
				glActiveTexture(GL_TEXTURE1);
				glDisable(GL_TEXTURE_2D);
				glActiveTexture(GL_TEXTURE0);
				glDisable(GL_TEXTURE_2D);
				glDepthFunc(GL_EQUAL);
			}
			mShdrMgr.BindProgram(mShdrPass2);
			mShdrMgr.PassUniform2fv("scale",1,Scale.v);
			UTIL_GL::SetBlend(GL_ONE, GL_ONE);
			mShdrMgr.PassSampler2D("tex",0,ringID);
			glColor3f(0,1,1);
			glBegin(GL_QUADS);
			glMultiTexCoord2f(GL_TEXTURE0,1,1); glVertex3fv((mPortal1Pos - r * 8 - u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,0,1); glVertex3fv((mPortal1Pos + r * 8 - u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,0,0); glVertex3fv((mPortal1Pos + r * 8 + u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,1,0); glVertex3fv((mPortal1Pos - r * 8 + u * 8).v);
			glEnd();
			glDisable(GL_TEXTURE_2D);
			mShdrMgr.BindProgram(0);
			glDisable(GL_BLEND);
			glColor3f(1,1,1);
			glDepthFunc(GL_LEQUAL);
		}
		else
		{
			UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			UTIL_GL::BindTexture(GL_TEXTURE_2D,portalID);
			glColor3f(1,1,1);
			glBegin(GL_QUADS);
			glTexCoord2f(1,1); glVertex3fv((mPortal1Pos - r * 8 - u * 8).v);
			glTexCoord2f(0,1); glVertex3fv((mPortal1Pos + r * 8 - u * 8).v);
			glTexCoord2f(0,0); glVertex3fv((mPortal1Pos + r * 8 + u * 8).v);
			glTexCoord2f(1,0); glVertex3fv((mPortal1Pos - r * 8 + u * 8).v);
			glEnd();
			glDepthFunc(GL_EQUAL);
			UTIL_GL::SetBlend(GL_ONE, GL_ONE);
			UTIL_GL::BindTexture(GL_TEXTURE_2D,ringID);
			glColor3f(0,1,1);
			glBegin(GL_QUADS);
			glTexCoord2f(1,1); glVertex3fv((mPortal1Pos - r * 8 - u * 8).v);
			glTexCoord2f(0,1); glVertex3fv((mPortal1Pos + r * 8 - u * 8).v);
			glTexCoord2f(0,0); glVertex3fv((mPortal1Pos + r * 8 + u * 8).v);
			glTexCoord2f(1,0); glVertex3fv((mPortal1Pos - r * 8 + u * 8).v);
			glEnd();
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			glColor3f(1,1,1);
			glDepthFunc(GL_LEQUAL);
		}
	}
	if(bRenderPortal2)
	{
		Vec3 r, u;
		if(mPortal2Normal.y <0)
		{
			r = Vec3(0,0,1);
			u = Vec3(1,0,0);
		}
		else if(mPortal2Normal.y >0)
		{
			r = Vec3(0,0,-1);
			u = Vec3(1,0,0);
		}
		else if(mPortal2Normal.x <0)
		{
			r = Vec3(0,0,-1);
			u = Vec3(0,1,0);
		}
		else if(mPortal2Normal.x >0)
		{
			r = Vec3(0,0,1);
			u = Vec3(0,1,0);
		}
		else if(mPortal2Normal.z <0)
		{
			r = Vec3(1,0,0);
			u = Vec3(0,1,0);
		}
		else if(mPortal2Normal.z >0)
		{
			r = Vec3(-1,0,0);
			u = Vec3(0,1,0);
		}
		if(bHiSpecs)
		{
			Vec2 Scale(0.1+sin(mTimer.GetTime()*3) * 0.1, 0.1+cos(mTimer.GetTime()*3) * 0.1);
			if(bRenderPortal1)
			{
				mShdrMgr.BindProgram(mShdrPass);
				mShdrMgr.PassUniform2fv("scale",1,Scale.v);
				UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				mShdrMgr.PassSampler2D("tex",0,portal2ID);
				mShdrMgr.PassSampler2D("tex2",1,portalID);
				glBegin(GL_QUADS);
				glMultiTexCoord2f(GL_TEXTURE0,1,1); 
				glMultiTexCoord2f(GL_TEXTURE1,1,1); glVertex3fv((mPortal2Pos - r * 8 - u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,0,1); 
				glMultiTexCoord2f(GL_TEXTURE1,0,1); glVertex3fv((mPortal2Pos + r * 8 - u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,0,0); 
				glMultiTexCoord2f(GL_TEXTURE1,0,0); glVertex3fv((mPortal2Pos + r * 8 + u * 8).v);
				glMultiTexCoord2f(GL_TEXTURE0,1,0); 
				glMultiTexCoord2f(GL_TEXTURE1,1,0); glVertex3fv((mPortal2Pos - r * 8 + u * 8).v);
				glEnd();
				glDisable(GL_TEXTURE_2D);
				mShdrMgr.BindProgram(0);
				glActiveTexture(GL_TEXTURE1);
				glDisable(GL_TEXTURE_2D);
				glActiveTexture(GL_TEXTURE0);
				glDisable(GL_TEXTURE_2D);
				glDepthFunc(GL_EQUAL);
			}
			mShdrMgr.BindProgram(mShdrPass2);
			mShdrMgr.PassUniform2fv("scale",1,Scale.v);
			UTIL_GL::SetBlend(GL_ONE, GL_ONE);
			mShdrMgr.PassSampler2D("tex",0,ringID);
			glColor3f(1,1,0);
			glBegin(GL_QUADS);
			glMultiTexCoord2f(GL_TEXTURE0,1,1); glVertex3fv((mPortal2Pos - r * 8 - u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,0,1); glVertex3fv((mPortal2Pos + r * 8 - u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,0,0); glVertex3fv((mPortal2Pos + r * 8 + u * 8).v);
			glMultiTexCoord2f(GL_TEXTURE0,1,0); glVertex3fv((mPortal2Pos - r * 8 + u * 8).v);
			glEnd();
			glDisable(GL_TEXTURE_2D);
			mShdrMgr.BindProgram(0);
			glDisable(GL_BLEND);
			glColor3f(1,1,1);
			glDepthFunc(GL_LEQUAL);
		}
		else
		{
			UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			UTIL_GL::BindTexture(GL_TEXTURE_2D,portalID);
			glColor3f(1,1,1);
			glBegin(GL_QUADS);
			glTexCoord2f(1,1); glVertex3fv((mPortal2Pos - r * 8 - u * 8).v);
			glTexCoord2f(0,1); glVertex3fv((mPortal2Pos + r * 8 - u * 8).v);
			glTexCoord2f(0,0); glVertex3fv((mPortal2Pos + r * 8 + u * 8).v);
			glTexCoord2f(1,0); glVertex3fv((mPortal2Pos - r * 8 + u * 8).v);
			glEnd();
			glDepthFunc(GL_EQUAL);
			UTIL_GL::SetBlend(GL_ONE, GL_ONE);
			UTIL_GL::BindTexture(GL_TEXTURE_2D,ringID);
			glColor3f(1,1,0);
			glBegin(GL_QUADS);
			glTexCoord2f(1,1); glVertex3fv((mPortal2Pos - r * 8 - u * 8).v);
			glTexCoord2f(0,1); glVertex3fv((mPortal2Pos + r * 8 - u * 8).v);
			glTexCoord2f(0,0); glVertex3fv((mPortal2Pos + r * 8 + u * 8).v);
			glTexCoord2f(1,0); glVertex3fv((mPortal2Pos - r * 8 + u * 8).v);
			glEnd();
			glDisable(GL_TEXTURE_2D);
			glDisable(GL_BLEND);
			glColor3f(1,1,1);
			glDepthFunc(GL_LEQUAL);
		}
	}

	glDisable(GL_DEPTH_TEST);

	if(bWonStage && bHiSpecs)
	{
		UTIL_GL::BindTexture(GL_TEXTURE_2D, iSceneID);
		glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,1024,512);
		glDisable(GL_TEXTURE_2D);
	}

	UTIL_GL::GL2D::SetOrtho(640,480);
	UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	UTIL_GL::BindTexture(GL_TEXTURE_2D, xhairID);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0); glVertex2f(320-16, 240-16);
	glTexCoord2f(1,0); glVertex2f(320+16, 240-16);
	glTexCoord2f(1,1); glVertex2f(320+16, 240+16);
	glTexCoord2f(0,1); glVertex2f(320-16, 240+16);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);


	if(bStartGame)
	{
		glColor3f(0,0,0);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(640,0);
		glVertex2f(640,240-iStartTime); glVertex2f(0,240-iStartTime);
		glVertex2f(0,240+iStartTime); glVertex2f(640,240+iStartTime);
		glVertex2f(640,480); glVertex2f(0,480);
		glEnd();
		float a;
		if(iStartTime<50)
			a = UTIL_Misc::Interpolate(0,1,(float)iStartTime/50);
		if(iStartTime>200)
			a = UTIL_Misc::Interpolate(1,0,(float)(iStartTime-200)/50);
		if(a>1)a=1;
		if(a<0)a=0;

		mFont1.SetAlignment(ALIGN_CENTER);
		mFont1.DisableShadow();
		mFont1.SetScale(1,1);
		mFont1.SetColor(1,1,1,1-a);
		switch(iStage)
		{
		case 0: 	mFont1.Print("CROSSING A CHASM...",320,240); break;
		case 1: 	mFont1.Print("TRANSFERING VELOCITY...",320,240); break;
		case 2: 	mFont1.Print("APPLYING GRAVITY...",320,240); break;
		case 3: 	mFont1.Print("PUTTING IT TOGETHER...",320,240); break;
		case 4: 	mFont1.Print("NOW ITS GETTING TOUGHER...",320,240); break;
		case 5: 	mFont1.Print("JUMP...",320,240); break;
		case 6: 	mFont1.Print("YOURE GETTING THE HANG OF THIS...",320,240); break;
		case 7: 	mFont1.Print("OH NOES!",320,240); break;
		}
		mFont1.SetColor(1,1,1,1);

	}
	if(bDead)
	{
		float a = UTIL_Misc::Interpolate(0,1,(float)iDeathTime/100);
		UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(1,0,0,a);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(640,0); glVertex2f(640,480); glVertex2f(0,480);
		glColor4f(0,0,0,1);
		glVertex2f(0,0); glVertex2f(640,0);
		glVertex2f(640,0+iDeathTime); glVertex2f(0,0+iDeathTime);
		glVertex2f(0,480-iDeathTime); glVertex2f(640,480-iDeathTime);
		glVertex2f(640,480); glVertex2f(0,480);
		glEnd();
		glDisable(GL_BLEND);
	}
	if(bWonStage)
	{
		float a = UTIL_Misc::Interpolate(0,1,(float)iWonTime/50);
		if(a>1)a=1;
		if(a<0)a=0;
		if(bHiSpecs)
		{
			mShdrMgr.BindProgram(mShdrGrey);
			mShdrMgr.PassUniform1f("interp",a);
			mShdrMgr.PassSampler2D("tex",0,iSceneID);
			glColor4f(1,1,1,a);
			glBegin(GL_QUADS);
			glTexCoord2f(0,1);glVertex2f(0,480-512);
			glTexCoord2f(1,1);glVertex2f(1024,480-512);
			glTexCoord2f(1,0);glVertex2f(1024,480);
			glTexCoord2f(0,0);glVertex2f(0,480);
			glEnd();
			mShdrMgr.BindProgram(0);
		}
		mFont1.SetAlignment(ALIGN_CENTER);
		mFont1.DisableShadow();
		mFont1.SetScale(1,1);
		mFont1.SetColor(1,1,1,1-a);
		mFont1.Print("WELL DONE",320,200);
		mFont1.Print("YOU MADE IT!",320,250);
		mFont1.SetColor(1,1,1,1);

		int time = iWonTime * 2;
		glColor4f(0,0,0,1);
		glBegin(GL_QUADS);
		glVertex2f(0,0); glVertex2f(640,0);
		glVertex2f(640,-150+time); glVertex2f(0,-150+time);
		glVertex2f(0,630-time); glVertex2f(640,630-time);
		glVertex2f(640,480); glVertex2f(0,480);
		glEnd();
		glColor4f(1,1,1,1);
	}

	float scroll = UTIL_Misc::Interpolate(gScroller,gOScroller,interp);

	mFont1.SetAlignment(ALIGN_LEFT);
	mFont1.DisableShadow();
	mFont1.EnableSineWave(scroll*10.01f,0.1f);
	mFont1.SetScale(1,1);
	mFont1.Print(scroller,scroll,450);
	mFont1.DisableSineWave();

	if(mbPaused)
	{
		UTIL_GL::SetBlend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glColor4f(0,0,0,0.5f);
		glBegin(GL_QUADS);
		glVertex2f(0,0);
		glVertex2f(640,0);
		glVertex2f(640,480);
		glVertex2f(0,480);
		glEnd();
		glColor4f(1,1,1,1);
		mFont1.SetAlignment(ALIGN_LEFT);
		mFont1.DisableShadow();
		mFont1.SetScale(0.75f,0.75f);

		mFont1.Print("MOUSE SENSI:",150,240);
		char sensi[64];
		sprintf(sensi,"%.2f",mfMouseSensi);
		mFont1.Print(sensi,450,240);
		mFont1.SetAlignment(ALIGN_CENTER);
		mFont1.Print("USE LEFT OR RIGHT TO CHANGE",320,280);
	}
}
void Engine::RenderMap(const float interp)
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	float LightPos[4]={0.007f, -0.1f, 0.06, 0};
	float LightPos2[4]={-0.04, 0.1f, -0.004f ,0};
	float amb[4]={0.95f,0.95f,0.95f,1};
	float dif[4]={0.6f,0.6f,0.6f,1};
	float spec[4]={0.45f,0.45f,0.45f,1};
	float blk[4]={0,0,0,1};
	float dif2[4]={0.26f,0.2f,0.16f,1};
	float spec2[4]={0.11f,0.05f,0.01f,1};
	glShadeModel(GL_SMOOTH);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, amb);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0,  GL_POSITION, LightPos);
	glLightfv(GL_LIGHT0,  GL_AMBIENT, spec);
	glLightfv(GL_LIGHT0,  GL_DIFFUSE, dif);
	glEnable(GL_LIGHT1);
	glLightfv(GL_LIGHT1,  GL_POSITION, LightPos2);
	glLightfv(GL_LIGHT1,  GL_AMBIENT, spec2);
	glLightfv(GL_LIGHT1,  GL_DIFFUSE, dif2);

	mMapObj.Render(true);
	if(!mPlatforms.empty())
	{
		std::list<Platform>::iterator plat;
		for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
		{
			(*plat).Render(interp);
		}
	}
	glDisable(GL_LIGHTING);
}

void Engine::ConvertMap()
{
	mapX = (mMapObj.mMaxs.x - mMapObj.mMins.x) / MAP_SCALE;
	mapY = (mMapObj.mMaxs.y - mMapObj.mMins.y) / MAP_SCALE;
	mapZ = (mMapObj.mMaxs.z - mMapObj.mMins.z) / MAP_SCALE;
	mPlatforms.clear();

	bTileMap = new int[mapX*mapY*mapZ];
	memset(bTileMap,0,sizeof(int)*mapX*mapY*mapZ);
	//shift all vertices by offset
	std::vector<Vec3>::iterator v;
	for(v=mMapObj.mVerts.begin(); v<mMapObj.mVerts.end(); v++)
	{
		(*v) -= mMapObj.mMins;
	}

	mOPlrPos = mPlrPos = Vec3(0,-20,0) - mMapObj.mMins;
	mPlrVel = Vec3(0,0,0);
	mfPlrGravity = 0;
	mPlrAngles = mOPlrAngles = Vec3(0,0,0);
	mPlrAngles.MakeVectors(mPlrFwd, mPlrRight, mPlrUp);
	mOPlrFwd = mPlrFwd;
	mOPlrRight = mPlrRight;
	mOPlrUp = mPlrUp;
	bOnPlatform = false;
	pRidingPlatform = NULL;
	bDead = false;
	iDeathTime = 0;
	bStartGame = true;
	iStartTime = 0;
	bWonStage = false;
	iWonTime = 0;
	bRenderPortal1 = false;
	bRenderPortal2 = false;
	iPortalDelayTime = 0;

	//fill the collision map
//	mMapObj.mUVs.clear();
	std::list<Face_t>::iterator f;
	int uvCounter = 0;
	for(f=mMapObj.mFaces.begin(); f!=mMapObj.mFaces.end(); f++,uvCounter++)
	{
		//create bounding box 
		Vec3 mins(99999,99999,99999);
		Vec3 maxs(-99999,-99999,-99999);
		for(int n=0; n<3; n++,uvCounter++)
		{
			//clamp the map vertices
/*			mMapObj.mVerts[(*f).vertex[n]] = Vec3(ceil(mMapObj.mVerts[(*f).vertex[n]].x),
				ceil(mMapObj.mVerts[(*f).vertex[n]].y),
				ceil(mMapObj.mVerts[(*f).vertex[n]].z));
*/			//create bounds for face
			if(mMapObj.mVerts[(*f).vertex[n]].x < mins.x)
				mins.x = mMapObj.mVerts[(*f).vertex[n]].x;
			if(mMapObj.mVerts[(*f).vertex[n]].y < mins.y)
				mins.y = mMapObj.mVerts[(*f).vertex[n]].y;
			if(mMapObj.mVerts[(*f).vertex[n]].z < mins.z)
				mins.z = mMapObj.mVerts[(*f).vertex[n]].z;

			if(mMapObj.mVerts[(*f).vertex[n]].x > maxs.x)
				maxs.x = mMapObj.mVerts[(*f).vertex[n]].x;
			if(mMapObj.mVerts[(*f).vertex[n]].y > maxs.y)
				maxs.y = mMapObj.mVerts[(*f).vertex[n]].y;
			if(mMapObj.mVerts[(*f).vertex[n]].z > maxs.z)
				maxs.z = mMapObj.mVerts[(*f).vertex[n]].z;

/*			(*f).uv[n] = uvCounter;
			//fix tex coords while here
/*			if(mMapObj.mVerts[(*f).vertex[0]].z == mMapObj.mVerts[(*f).vertex[1]].z == mMapObj.mVerts[(*f).vertex[2]].z)
				mMapObj.mUVs.push_back(Vec2(mMapObj.mVerts[(*f).vertex[n]].x, mMapObj.mVerts[(*f).vertex[n]].y) / MAP_SCALE);
			else if(mMapObj.mVerts[(*f).vertex[0]].x == mMapObj.mVerts[(*f).vertex[1]].x == mMapObj.mVerts[(*f).vertex[2]].x)
				mMapObj.mUVs.push_back(Vec2(mMapObj.mVerts[(*f).vertex[n]].z, mMapObj.mVerts[(*f).vertex[n]].y) / MAP_SCALE);
			else*/
//				mMapObj.mUVs.push_back(Vec2(mMapObj.mVerts[(*f).vertex[n]].x, mMapObj.mVerts[(*f).vertex[n]].z) / MAP_SCALE);
		}
		//convert it to tile map coords
		mins /= MAP_SCALE;
		maxs /= MAP_SCALE;
		//push the box in the normals opposite direction
		mins -= mMapObj.mNormals[(*f).normal[1]] * 0.15f;
		maxs -= mMapObj.mNormals[(*f).normal[1]] * 0.15f;

		//fill the tilemap
		int minX = mins.x;
		int minY = mins.y;
		int minZ = mins.z;
		int maxX = maxs.x;
		int maxY = maxs.y;
		int maxZ = maxs.z;
		if(maxX==minX)
			maxX = minX+1;
		if(maxY==minY)
			maxY = minY+1;
		if(maxZ==minZ)
			maxZ = minZ+1;

		for(int X=minX; X<maxX; X++)
		for(int Y=minY; Y<maxY; Y++)
		for(int Z=minZ; Z<maxZ; Z++)
		{
			if(X<0 || Y<0 || Z<0 || X>=mapX || Y>=mapY || Z>=mapZ)
				continue;

			int blockType = 0;
			char name[64];
			if(mTexMgr.GetTextureName((*f).mattID, name))
			{
				if(!stricmp("tile1.png",name))
					blockType = PORTAL_TILE;
				else if(!stricmp("tile3.png",name))
					blockType = PORTAL_TILE;
				else if(!stricmp("tile4.png",name))
					blockType = PORTAL_TILE;
				else if(!stricmp("Exit.png",name))
					blockType = EXIT_TILE;
				else
					blockType = NON_PORTAL_TILE;
			}
			bTileMap[Z*mapY*mapX+Y*mapX+X] = blockType;
		}

		//create platforms
		char name[64];
		if(mTexMgr.GetTextureName((*f).mattID, name))
		{
			if(!stricmp("Plat1.png",name))
			{
				(*f).mattID = gpEng->mTexMgr.LoadTexture(false,"Data","Plat.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT);
				mins += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				maxs += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				Platform plat(mMapObj.mNormals[(*f).normal[1]], mins*MAP_SCALE, maxs*MAP_SCALE,  1 * (1.0/TIME_STEP), (*f).mattID);
				mPlatforms.push_back(plat);
				gLog.OutPut("Created platform!\n");
			}
			if(!stricmp("Plat2.png",name))
			{
				(*f).mattID = gpEng->mTexMgr.LoadTexture(false,"Data","Plat.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT);
				mins += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				maxs += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				Platform plat(mMapObj.mNormals[(*f).normal[1]], mins*MAP_SCALE, maxs*MAP_SCALE,  2 * (1.0/TIME_STEP), (*f).mattID);
				mPlatforms.push_back(plat);
				gLog.OutPut("Created platform!\n");
			}
			if(!stricmp("Plat3.png",name))
			{
				(*f).mattID = gpEng->mTexMgr.LoadTexture(false,"Data","Plat.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT);
				mins += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				maxs += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				Platform plat(mMapObj.mNormals[(*f).normal[1]], mins*MAP_SCALE, maxs*MAP_SCALE,  3 * (1.0/TIME_STEP), (*f).mattID);
				mPlatforms.push_back(plat);
				gLog.OutPut("Created platform!\n");
			}
			if(!stricmp("Plat4.png",name))
			{
				(*f).mattID = gpEng->mTexMgr.LoadTexture(false,"Data","Plat.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT);
				mins += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				maxs += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				Platform plat(mMapObj.mNormals[(*f).normal[1]], mins*MAP_SCALE, maxs*MAP_SCALE,  4 * (1.0/TIME_STEP), (*f).mattID);
				mPlatforms.push_back(plat);
				gLog.OutPut("Created platform!\n");
			}
			if(!stricmp("Plat5.png",name))
			{
				(*f).mattID = gpEng->mTexMgr.LoadTexture(false,"Data","Plat.png",GL_LINEAR, GL_LINEAR, GL_REPEAT, GL_REPEAT);
				mins += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				maxs += mMapObj.mNormals[(*f).normal[1]] * 0.3f;
				Platform plat(mMapObj.mNormals[(*f).normal[1]], mins*MAP_SCALE, maxs*MAP_SCALE,  5 * (1.0/TIME_STEP), (*f).mattID);
				mPlatforms.push_back(plat);
				gLog.OutPut("Created platform!\n");
			}
		}
	}

}
int Engine::GetTileState(Vec3 p)
{
	p /= MAP_SCALE;
	int x = p.x;
	int y = p.y;
	int z = p.z;

	if(x<0 || y<0 || z<0 || x>=mapX || y>=mapY || z>=mapZ)
		return 99;

	return bTileMap[z*mapY*mapX+y*mapX+x];
}
int Engine::ShootRay(Vec3 p, Vec3 ray, Vec3& out, Vec3& normal)
{
	out = p;
	const int maxlength = 500;
	for(int n=1; n<maxlength; n++)
	{
		out += ray * 0.5f;
		//make sure it doesnt hit a platform first
		if(!mPlatforms.empty())
		{
			std::list<Platform>::iterator plat;
			for(plat=mPlatforms.begin(); plat!=mPlatforms.end(); plat++)
			{
				if((*plat).CheckPointInPlat(out))
				{
					//gLog.OutPut("Cant create portals on a platform!\n");
					return NON_PORTAL_TILE;
				}
			}
		}

		int tile;
		if((tile=GetTileState(out)))
		{
			int bX = (int)out.x / MAP_SCALE;
			int bY = (int)out.y / MAP_SCALE;
			int bZ = (int)out.z / MAP_SCALE;

			int cX,cY,cZ;
			while(GetTileState(out))
			{
				out -= ray *0.1f;
				cX = (int)out.x / MAP_SCALE;
				cY = (int)out.y / MAP_SCALE;
				cZ = (int)out.z / MAP_SCALE;
			}

			if(cX<bX)
				normal = Vec3(-1,0,0);
			if(cX>bX)
				normal = Vec3(1,0,0);
			if(cZ<bZ)
				normal = Vec3(0,0,-1);
			if(cZ>bZ)
				normal = Vec3(0,0,1);
			if(cY<bY)
				normal = Vec3(0,-1,0);
			if(cY>bY)
				normal = Vec3(0,1,0);
			return tile;
		}

	}

	return 0;
}
bool Engine::LoadStage()
{
	char map[64];
/*	switch(iStage)
	{
	case 0:	strcpy(map,"map1.obj");break;
	case 1:	strcpy(map,"map2.obj");break;
	case 2:	strcpy(map,"map3.obj");break;
	case 3:	strcpy(map,"map4.obj");break;
	case 4:	strcpy(map,"map5.obj");break;
	case 5:	strcpy(map,"map6.obj");break;
	case 6:	strcpy(map,"map7.obj");break;
	case 7:	strcpy(map,"map8.obj");break;
	}*/
	sprintf(map,"map%i.obj",iStage+1);
	if(!mMapObj.Load("Data",map,MAP_SCALE,true))
		return false;
	ConvertMap();

	return true;
}