#pragma once
#include "UTIL_SDL.h"
#include "UTIL_OpenGL.h"
#include "UTIL_Misc.h"
#include "Timer.h"
#include "FileIO.h"
#include "Button.h"
#include "Log.h"
#include "Vector.h"
#include "Font.h"
#include <vector>
#include <list>
#include <string>
#include <algorithm>
#include "TextureManager.h"
#include "OBJ.h"
#include "ShaderMgr.h"
#include "Shaders.h"
#include "Platform.h"

const float TIME_STEP = 0.02f;
#define MAP_SCALE 10
#define PLAYER_HEIGHT 12
#define DEATH_SPEED 2.5f

#define NON_PORTAL_TILE 99
#define PORTAL_TILE 1
#define EXIT_TILE 64

class Engine
{
public:
	double accumulator;

	Engine(int w, int h, bool fscreen);
	~Engine();
	int winWidth, winHeight;

	void InitDevil();
	void Pump();

	void Update();
	void Render(const float interp);
	bool InitializeExtensions();
	bool LoadShader(char* name, char* szVp, char* szFp, int& outShdr);


	void KeyStateAlltered(int key, bool state);
	void TextInput(SDL_keysym key);
	void KeysPostUpdate();
	void CheckJoystickMotion(SDL_Event event);
	void SaveInput();
	void LoadInput();
	Button KEY_MOUSE1, KEY_MOUSE2, KEY_MOUSE3, KEY_QUIT, KEY_PAUSE;
	Button KEY_LEFT, KEY_RIGHT, KEY_FORWARD, KEY_BACK, KEY_JUMP;
	float fJoystickSensi;
	bool bCfgJoystick;
	Button* pCfgButton;
	int iControlMethod;
	int MouseX, MouseY;
	bool bTextInput;
	char szInputText[6];
	int iTextIdx, iTextMaxLength;
	Vec2 mMouseMotion;
	float mfMouseSensi;
	int iOption;	

	bool mbQuit,mbPaused;
	Timer mTimer;
	Font mFont1;
	TextureManager mTexMgr;
	ShaderMgr mShdrMgr;


	float mInterp;

	unsigned int miInternalClock;

	//map
	int mapX, mapY, mapZ;
	int GetTileState(Vec3 p);
	int ShootRay(Vec3 p, Vec3 ray, Vec3& out, Vec3& normal);//return tile type
	void RenderMap(const float interp);
	OBJ mMapObj;
	void ConvertMap();
	int* bTileMap;

	int iStage;
	bool LoadStage();
	//player
	Vec3 mPlrPos, mOPlrPos;
	Vec3 mPlrVel;
	Vec3 mPlrFwd, mOPlrFwd;
	Vec3 mPlrRight, mOPlrRight;
	Vec3 mPlrUp, mOPlrUp;
	Vec3 mPlrAngles, mOPlrAngles;
	float mfPlrGravity;
	bool bOnGround;
	bool bPortalFriction;
	bool bOnPlatform;
	Platform* pRidingPlatform;
	bool bDead;
	int iDeathTime;
	bool bStartGame;
	int iStartTime;
	bool bWonStage;
	int iWonTime;
	//portals
	Vec3 mPortal1Pos, mPortal1Normal;
	Vec3 mPortal2Pos, mPortal2Normal;
	bool bRenderPortal1;
	bool bRenderPortal2;
	int iPortalDelayTime;
	//gfx
	unsigned int xhairID;
	unsigned int portalID, ringID;
	unsigned int portal1ID, portal2ID;
	unsigned int iSceneID;
	//shaders
	int mShdrPass;
	int mShdrPass2;
	int mShdrGrey;
	bool bHiSpecs;

	//platforms
	std::list<Platform> mPlatforms;
};
extern Engine* gpEng;