#include "Engine.h"
#include "SDL_KeyMap.h"
#include "Serializer.h"
/*==========================================*/
/*=  a key state has changed               =*/
/*==========================================*/
void Engine::KeyStateAlltered(int key, bool state)
{
	if(bCfgJoystick)
	{
		if(key == SDLK_ESCAPE)
		{
			bCfgJoystick = false;
			return;
		}
		if(key<JOYSTICK_BUTTON || !state)
			return;
		pCfgButton->secondKey = key;
		iOption++;
		return;
	}
	if(key == KEY_MOUSE3.primaryKey || key == KEY_MOUSE3.secondKey)
	{	KEY_MOUSE3.Update(state);	return; }
	if(key == KEY_MOUSE1.secondKey)
	{	KEY_MOUSE1.Update(state);	return; }
	if(key == KEY_LEFT.primaryKey || key == KEY_LEFT.secondKey)
	{	KEY_LEFT.Update(state);	return; }
	if(key == KEY_RIGHT.primaryKey || key == KEY_RIGHT.secondKey)
	{	KEY_RIGHT.Update(state);	return; }
	if(key == KEY_FORWARD.primaryKey || key == KEY_FORWARD.secondKey)
	{	KEY_FORWARD.Update(state);	return; }
	if(key == KEY_BACK.primaryKey || key == KEY_BACK.secondKey)
	{	KEY_BACK.Update(state);	return; }
	if(key == KEY_JUMP.primaryKey || key == KEY_JUMP.secondKey)
	{	KEY_JUMP.Update(state);	return; }

	switch(key)
	{
	case SDLK_ESCAPE: KEY_QUIT.Update(state); break;
	case SDLK_PAUSE: KEY_PAUSE.Update(state); break;
	case 1: KEY_MOUSE1.Update(state); break;
	case 2: KEY_MOUSE2.Update(state); break;
	case 3: KEY_MOUSE3.Update(state); break;

	default:
		break;
	}
}
/*==========================================*/
/*=post update on key (allows more control)=*/
/*==========================================*/
void Engine::KeysPostUpdate()
{
	KEY_QUIT.PostUpdate();
	KEY_PAUSE.PostUpdate();
	KEY_MOUSE1.PostUpdate();
	KEY_MOUSE2.PostUpdate();
	KEY_MOUSE3.PostUpdate();
	KEY_LEFT.PostUpdate();
	KEY_RIGHT.PostUpdate();
	KEY_FORWARD.PostUpdate();
	KEY_BACK.PostUpdate();
	KEY_JUMP.PostUpdate();
}

void Engine::SaveInput()
{
	DeleteFile("Input.dat");
	gSerializer.PutComment("Input.dat","Input Configuration");
	gSerializer.ReadVariable("Input.dat","PAD_SHOOT",KEY_MOUSE1.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_SHOOT2",KEY_MOUSE3.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_LEFT",KEY_LEFT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_RIGHT",KEY_RIGHT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_UP",KEY_FORWARD.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_DOWN",KEY_BACK.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_JUMP",KEY_JUMP.secondKey);
	gSerializer.ReadVariable("Input.dat","JoySensi",mfMouseSensi);
}
void Engine::LoadInput()
{
	fJoystickSensi = 2.5f;
	KEY_MOUSE3.primaryKey = -1;
	KEY_MOUSE2.secondKey = -1;
	KEY_MOUSE1.secondKey = -1;
	KEY_LEFT.primaryKey = SDLK_a;
	KEY_LEFT.secondKey = SDLK_LEFT;
	KEY_RIGHT.primaryKey = SDLK_d;
	KEY_RIGHT.secondKey = SDLK_RIGHT;
	KEY_FORWARD.primaryKey = SDLK_w;
	KEY_FORWARD.secondKey = SDLK_UP;
	KEY_BACK.primaryKey = SDLK_s;
	KEY_BACK.secondKey = SDLK_DOWN;
	KEY_JUMP.primaryKey = SDLK_SPACE;
	KEY_JUMP.secondKey = SDLK_LCTRL;
	bCfgJoystick = false;
	CFileIO fIO;
	if(fIO.GetFile("Input.dat","rt"))
	{
	gSerializer.ReadVariable("Input.dat","PAD_SHOOT",KEY_MOUSE1.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_SHOOT2",KEY_MOUSE3.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_LEFT",KEY_LEFT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_RIGHT",KEY_RIGHT.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_UP",KEY_FORWARD.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_DOWN",KEY_BACK.secondKey);
	gSerializer.ReadVariable("Input.dat","PAD_JUMP",KEY_JUMP.secondKey);
	gSerializer.ReadVariable("Input.dat","JoySensi",mfMouseSensi);
	}
}

//joystick
void Engine::CheckJoystickMotion(SDL_Event event)
{
//	gLog.OutPut("CheckJoystick START\n");
	if(event.jaxis.axis == 0)//horizontal?
	{
		int axis;
		if(event.jaxis.value<0)
			axis=event.jaxis.value;
		else
			axis=event.jaxis.value;

		if(axis<0)
			mMouseMotion.x = -pow(((float)(axis*0.9f)/32768.0f)*fJoystickSensi,2);
		if(axis>0)
			mMouseMotion.x = pow(((float)(axis*0.9f)/32768.0f)*fJoystickSensi,2);
	}
	if(event.jaxis.axis == 1)//Vertical?
	{
		int axis;
		if(event.jaxis.value<0)
			axis=event.jaxis.value;
		else
			axis=event.jaxis.value;
		

		if(axis<0)
			mMouseMotion.y = -pow(((float)(axis*0.9f)/32768.0f)*fJoystickSensi,2);
		if(axis>0)
			mMouseMotion.y = pow(((float)(axis*0.9f)/32768.0f)*fJoystickSensi,2);
	}
//	gLog.OutPut("CheckJoystick END\n");
}

void Engine::TextInput(SDL_keysym key)
{
	//text input
	if(bTextInput)
	{
//		Mix_PlayChannel(-1, sndUIMove, 0);
		if(key.sym == SDLK_RETURN || key.sym == SDLK_ESCAPE)
		{
			bTextInput = false;
			SDL_EnableUNICODE(0);
			szInputText[iTextIdx] = '\0';
			return;
		}
		else if(key.sym == SDLK_BACKSPACE)
		{
			iTextIdx>0?iTextIdx--:iTextIdx=0;
			szInputText[iTextIdx] = '_';
			return;
		}
		if(key.unicode == 0)
			return;
		if((key.unicode & 0xFF80) == 0 )
			szInputText[iTextIdx] = toupper(key.unicode & 0x7F);
		iTextIdx++;

		//szInputText[iTextIdx] = ;
		if(iTextIdx>=iTextMaxLength)
		{
			bTextInput = false;
			SDL_EnableUNICODE(0);
		}
	}
}