#include "Platform.h"
#include "Engine.h"

Platform::Platform(Vec3 n, Vec3 mins, Vec3 maxs, int delay, unsigned int ID)
{
	mDelay = delay + 150;
	imgID = ID;
	mMins = mins;
	mMaxs = maxs;
	mNormal = n;

	Vec3 dir = ((mMaxs-mMins)/2);
	mStartPos = mOCenter = mCenter = mMins + dir;
	mCenter += mNormal * 0.1f;
	//create new bounds
	if(mCenter.x < mMins.x)
		mMins.x = mCenter.x;
	if(mCenter.y < mMins.y)
		mMins.y = mCenter.y;
	if(mCenter.z < mMins.z)
		mMins.z = mCenter.z;

	if(mCenter.x > mMaxs.x)
		mMaxs.x = mCenter.x;
	if(mCenter.y > mMaxs.y)
		mMaxs.y = mCenter.y;
	if(mCenter.z > mMaxs.z)
		mMaxs.z = mCenter.z;
	bMoving = false;
}
void Platform::Update()
{
	if(!bMoving&&mDelay>0)
		mDelay--;
	if(mDelay>0)
		return;
	if(mDelay==0)
	{
		bMoving = true;
		mDelay = -1;
	}

	if(!bMoving)
		return;

	mOCenter = mCenter;
	mCenter += mNormal * PLATFORM_SPEED;
	if(gpEng->GetTileState(mCenter))
		bMoving = false;

	if(gpEng->bRenderPortal1)
	{
		if(CheckPointInPlat(gpEng->mPortal1Pos))
			gpEng->bRenderPortal1 = false;
	}
	if(gpEng->bRenderPortal2)
	{
		if(CheckPointInPlat(gpEng->mPortal2Pos))
			gpEng->bRenderPortal2 = false;
	}
	//check see if the player moved inside me during the time that I moved.. if so push him out
	if(CheckPointInPlat(gpEng->mPlrPos-mNormal*1.5f) || CheckPointInPlat(gpEng->mPlrPos-mNormal*1.5f + Vec3(0,PLAYER_HEIGHT-1,0)))
	{
		gpEng->mPlrPos += mNormal * PLATFORM_SPEED;
		gpEng->mOPlrPos += mNormal * PLATFORM_SPEED;

		if(mNormal.y == 1)
		if(gpEng->bOnGround)
		{
			gpEng->bDead = true;
			gpEng->iDeathTime = 0;
			gpEng->mOPlrPos = gpEng->mPlrPos;
			gpEng->mOPlrAngles = gpEng->mPlrAngles;
		}
		//check see if we are crushing against a wall
		if(mNormal.y == 0)
		{
			for(int n=1; n<PLAYER_HEIGHT; n++)
			if(gpEng->GetTileState(gpEng->mPlrPos + Vec3(0,1,0) * n + mNormal * 1.5f))
			{
				gpEng->bDead = true;
				gpEng->iDeathTime = 0;
				gpEng->mOPlrPos = gpEng->mPlrPos;
				gpEng->mOPlrAngles = gpEng->mPlrAngles;
				break;
			}
		}
	}
	//create new bounds
	if(mCenter.x < mMins.x)
		mMins.x = mCenter.x;
	if(mCenter.y < mMins.y)
		mMins.y = mCenter.y;
	if(mCenter.z < mMins.z)
		mMins.z = mCenter.z;

	if(mCenter.x > mMaxs.x)
		mMaxs.x = mCenter.x;
	if(mCenter.y > mMaxs.y)
		mMaxs.y = mCenter.y;
	if(mCenter.z > mMaxs.z)
		mMaxs.z = mCenter.z;

}

void Platform::Render(const float interp)
{
	//render a cube
	UTIL_GL::BindTexture(GL_TEXTURE_2D, imgID);
	glBegin(GL_QUADS);
	//front face
	glTexCoord2f(0,0);	glVertex3f(mMins.x, mMins.y, mMins.z);
	glTexCoord2f(1,0);	glVertex3f(mMaxs.x, mMins.y, mMins.z);
	glTexCoord2f(1,1);	glVertex3f(mMaxs.x, mMaxs.y, mMins.z);
	glTexCoord2f(0,1);	glVertex3f(mMins.x, mMaxs.y, mMins.z);
	//back face
	glTexCoord2f(0,0);	glVertex3f(mMaxs.x, mMins.y, mMaxs.z);
	glTexCoord2f(1,0);	glVertex3f(mMins.x, mMins.y, mMaxs.z);
	glTexCoord2f(1,1);	glVertex3f(mMins.x, mMaxs.y, mMaxs.z);
	glTexCoord2f(0,1);	glVertex3f(mMaxs.x, mMaxs.y, mMaxs.z);
	//left face
	glTexCoord2f(0,0);	glVertex3f(mMins.x, mMins.y, mMins.z);
	glTexCoord2f(1,0);	glVertex3f(mMins.x, mMins.y, mMaxs.z);
	glTexCoord2f(1,1);	glVertex3f(mMins.x, mMaxs.y, mMaxs.z);
	glTexCoord2f(0,1);	glVertex3f(mMins.x, mMaxs.y, mMins.z);
	//right face
	glTexCoord2f(0,0);	glVertex3f(mMaxs.x, mMins.y, mMaxs.z);
	glTexCoord2f(1,0);	glVertex3f(mMaxs.x, mMins.y, mMins.z);
	glTexCoord2f(1,1);	glVertex3f(mMaxs.x, mMaxs.y, mMins.z);
	glTexCoord2f(0,1);	glVertex3f(mMaxs.x, mMaxs.y, mMaxs.z);
	//top face
	glTexCoord2f(0,0);	glVertex3f(mMins.x, mMins.y, mMins.z);
	glTexCoord2f(1,0);	glVertex3f(mMaxs.x, mMins.y, mMins.z);
	glTexCoord2f(1,1);	glVertex3f(mMaxs.x, mMins.y, mMaxs.z);
	glTexCoord2f(0,1);	glVertex3f(mMins.x, mMins.y, mMaxs.z);
	//bottom face
	glTexCoord2f(0,0);	glVertex3f(mMaxs.x, mMaxs.y, mMins.z);
	glTexCoord2f(1,0);	glVertex3f(mMins.x, mMaxs.y, mMins.z);
	glTexCoord2f(1,1);	glVertex3f(mMins.x, mMaxs.y, mMaxs.z);
	glTexCoord2f(0,1);	glVertex3f(mMaxs.x, mMaxs.y, mMaxs.z);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}

bool Platform::CheckPointInPlat(Vec3 p)
{
	if(p.x< mMins.x || p.x>mMaxs.x)
		return false;
	if(p.y< mMins.y || p.y>mMaxs.y)
		return false;
	if(p.z< mMins.z || p.z>mMaxs.z)
		return false;

	return true;
}
